from bs4 import BeautifulSoup
from markdown import markdown
import click
import yaml
import re


def separate_blogpost(content): # <1>
    frontmatter = {}
    if content.startswith('---'):
        match = re.search(
            r'^(?P<yamlblock>---(?P<yaml>.*?)---[ \t]*\r?\n)', # <2>
            content, re.DOTALL)
        if match:
            frontmatter = yaml.load(match.group('yaml')) # <3>
            content = content[match.end('yamlblock'):]

    return (frontmatter, content)


@click.command()
@click.option('--dictionary', '-d', type=click.File('r'))
@click.option('--warnings', '-w', default=0, type=int) # <4>
@click.argument('input_file', type=click.File('r'))
def check_understandability(dictionary, warnings, input_file):
    words = [w.strip().lower() for w in dictionary.readlines()] # <5>

    frontmatter, content = separate_blogpost(input_file.read())
    if frontmatter.get('Approved', False): # <6>
        print('Understandability: OK')
        raise SystemExit(0)
    
    html = markdown(content) # <7>
    text = ''.join(BeautifulSoup(html, 'html.parser').findAll(text=True)) # <8>

    regular_expression = re.compile(
        r"^\W?(?P<word>(\w|-)+)('s)?\W*$") # <9>
    bad_words = set()
    for input_word in text.split():
        match = regular_expression.match(input_word) # <10>
        if match is None:
            continue
        clean_word = match.group('word') # <11>
        if clean_word.lower() not in words: # <12>
            bad_words.add(input_word)

    if len(bad_words) > warnings: # <13>
        print('WARNING: {count} words may not fit in blog post '\
              '{words}'.format(count=len(bad_words), words=list(bad_words)))
        raise SystemExit(1)

    print('Understandability: OK') # <14>
    raise SystemExit(0)


if __name__ == '__main__':
    try:
        check_understandability()
    except Exception as e:
        print('UNKNOWN: {0}'.format(e))
        raise SystemExit(3)
