from os import environ as env
from ftplib import FTP
import hashlib
import click
import gpg


def extract_checksum(ciphertext, output): # <1>
    with gpg.Context() as gpgcontext: # <2>
        results = gpgcontext.decrypt(ciphertext,
                                     passphrase=env.get('PASSPHRASE'))
        manifest = results[0].decode()
        
        volumes_and_hashes = filter( # <3>
            lambda l: l.startswith('Volume') or l.lstrip().startswith('Hash'),
            manifest.splitlines())

        iterator_copies = [iter(volumes_and_hashes)] * 2 # <4>
        for (volume, hashvalue) in zip(*iterator_copies): # <5>
            volume = volume.lstrip('Volume ').rstrip(':')
            hashvalue = hashvalue.lstrip().lstrip('Hash SHA1 ')
            output.append((volume, hashvalue))


@click.command()
@click.option('--critical', '-c', type=int)
@click.argument('ftpserver')
def check_recoverability(critical, ftpserver):
    ftp = FTP(ftpserver) # <6>
    ftp.login(env.get('FTP_USER', 'ftpuser'),
              env.get('FTP_PASSWORD', 'ftppass'))

    filelist = ftp.nlst()
    manifests = sorted(filter(lambda filename: 'manifest' in filename, # <7>
                              filelist))

    manifest = manifests.pop()
    prefix = manifest.rstrip('.manifest.gpg') # <8>
    volumehashes = []
    ftp.retrbinary('RETR {filename}'.format(filename=manifest), # <9>
                   lambda content: extract_checksum(content, volumehashes))

    invalid_hashes = []
    for (volume, hashvalue) in volumehashes:
        volumefiles = filter( # <10>
            lambda filename: '{}.vol{}.'.format(prefix, volume) in filename,
            filelist)
        for volumefile in volumefiles:
            ftp.retrbinary(
                'RETR {filename}'.format(filename=volumefile), # <11>
                lambda content: invalid_hashes.append( # <12>
                    hashlib.sha1(content).hexdigest() != hashvalue))

    invalid_count = sum(invalid_hashes) # <13>
    
    if invalid_count > critical:
        print('CRITICAL: Recoverability ({count} volumes invalid)'.format(
            count=invalid_count))
        raise SystemExit(2)

    print('OK: Recoverability')
    raise SystemExit(0)


if __name__ == '__main__':
    try:
        check_recoverability()
    except Exception as e:
        print('UNKNOWN: {0}'.format(e))
        raise SystemExit(3)
